/********************************** (C) COPYRIGHT *******************************
 * File Name          : ch32v20x_it.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2023/12/29
 * Description        : Main Interrupt Service Routines.
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
#include "ch32v20x_it.h"
#include "board.h"
#include <rtthread.h>
void NMI_Handler(void) __attribute__((interrupt()));
void HardFault_Handler(void) __attribute__((interrupt()));
void EXTI2_IRQHandler(void) __attribute__((interrupt()));
void EXTI3_IRQHandler(void) __attribute__((interrupt()));

/*********************************************************************
 * @fn      NMI_Handler
 *
 * @brief   This function handles NMI exception.
 *
 * @return  none
 */
void NMI_Handler(void)
{
    GET_INT_SP();
    rt_interrupt_enter();
    rt_kprintf(" NMI Handler\r\n");
    rt_interrupt_leave();
    FREE_INT_SP();
}

/*********************************************************************
 * @fn      HardFault_Handler
 *
 * @brief   This function handles Hard Fault exception.
 *
 * @return  none
 */
void HardFault_Handler(void)
{
    GET_INT_SP();
    rt_interrupt_enter();
    rt_kprintf(" hardfult\r\n");
    rt_interrupt_leave();
    FREE_INT_SP();

}

__attribute__( ( always_inline ) ) RV_STATIC_INLINE uint32_t NVIC_Get_GISR(void)
{
    return (uint32_t) (NVIC->GISR);
}
__attribute__( ( always_inline ) ) RV_STATIC_INLINE uint32_t get_intsyscr(void)
{
    uint32_t value;
    asm("csrr %0," "0x804": "=r"(value));
    return (value);
}
__attribute__( ( always_inline ) ) RV_STATIC_INLINE uint32_t NVIC_Get_IPR1(void)
{
    return(uint32_t)(NVIC->IPR[0]);
}
__attribute__( ( always_inline ) ) RV_STATIC_INLINE uint32_t NVIC_Get_IPR2(void )
{
    return(uint32_t)(NVIC->IPR[1]);
}
__attribute__( ( always_inline ) ) RV_STATIC_INLINE uint32_t NVIC_Get_IPR3(void )
{
    return(uint32_t)(NVIC->IPR[2]);
}
__attribute__( ( always_inline ) ) RV_STATIC_INLINE uint32_t NVIC_Get_IPR4(void )
{
    return(uint32_t)(NVIC->IPR[3]);
}

void delay(uint32_t num)
{
    while(num>0)
    {
        asm("nop");
        num--;
    }
}


void EXTI2_IRQHandler(void)
{
    GET_INT_SP();
    rt_interrupt_enter();  /* enter interrupt */
    if(EXTI_GetITStatus(EXTI_Line2)!= RESET)
    {
        rt_kprintf("\r\n");
        rt_kprintf("--into exti2 handler----\r\n");
        rt_kprintf("NVIC->GISR: 0x%08X\r\n", NVIC_Get_GISR());
        rt_kprintf("INTSYSCR: 0x%08X\r\n", get_intsyscr());
        rt_kprintf("MSTATUS: 0x%08X\r\n", __get_MSTATUS());
        rt_kprintf("PFIC_IPR1: 0x%08X\r\n", NVIC_Get_IPR1());
        SysTick->CTLR=0;
        NVIC_DisableIRQ(SysTicK_IRQn);
        NVIC_ClearPendingIRQ(SysTicK_IRQn);
        rt_kprintf("PFIC_IPR1: 0x%08X\r\n", NVIC_Get_IPR1());
        rt_kprintf("PFIC_IPR2: 0x%08X\r\n",NVIC_Get_IPR2());
        rt_kprintf("PFIC_IPR3: 0x%08X\r\n",NVIC_Get_IPR3());
        rt_kprintf("PFIC_IPR4: 0x%08X\r\n",NVIC_Get_IPR4());
        delay(150000000);
        rt_kprintf("exit from exti2\r\n");
        EXTI_ClearITPendingBit(EXTI_Line2);
    }
    rt_interrupt_leave();   /* leave interrupt */
    FREE_INT_SP();
}

void EXTI3_IRQHandler(void)
{
    GET_INT_SP();
    rt_interrupt_enter();  /* enter interrupt */
    if(EXTI_GetITStatus(EXTI_Line3)!= RESET)
    {
        rt_kprintf("\r\n");
        rt_kprintf("--into exti3 handler----\r\n");
        rt_kprintf("NVIC->GISR: 0x%08X\r\n", NVIC_Get_GISR());
        rt_kprintf("INTSYSCR: 0x%08X\r\n", get_intsyscr());
        rt_kprintf("MSTATUS: 0x%08X\r\n", __get_MSTATUS());
        rt_kprintf("PFIC_IPR1: 0x%08X\r\n", NVIC_Get_IPR1());
        SysTick->CTLR=0;
        NVIC_DisableIRQ(SysTicK_IRQn);
        NVIC_ClearPendingIRQ(SysTicK_IRQn);
        rt_kprintf("PFIC_IPR1: 0x%08X\r\n", NVIC_Get_IPR1());
        rt_kprintf("PFIC_IPR2: 0x%08X\r\n",NVIC_Get_IPR2());
        rt_kprintf("PFIC_IPR3: 0x%08X\r\n",NVIC_Get_IPR3());
        rt_kprintf("PFIC_IPR4: 0x%08X\r\n",NVIC_Get_IPR4());
        rt_kprintf("waitting for exti2...\r\n");
        delay(150000000);
        rt_kprintf("exit from exti3\r\n");
        EXTI_ClearITPendingBit(EXTI_Line3);
    }
    rt_interrupt_leave();   /* leave interrupt */
    FREE_INT_SP();
}
